#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2023 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################
import time
from pathlib import Path
from typing import List, Callable, Optional, Any
from math import fabs
import concurrent.futures
import wx
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import MeasDlgThread
import nardascripting.base.toolbox as tb
from . import geocalc
from . import filehandling as fh
from . import nettools as nt
from .dfadata import SettingsData
from .navdev import INavDevBase
from .gui.dfadialogs import DFAInfoDlg


class DFAutopilot(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "DF Autopilot"
        self._scr_title = "Start DF Autopilot V0.9"
        self._scr_description = "Controls the navigation system according to the calculated localization result."
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 1
        self._nsl_executed_behavior = NSL_Executed_Behaviors.SHOW_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        # self.my_variable = None
        self.navidev = ''

    def _run_script(self, args):
        """ Script main function

        This method is called when a user clicks on the corresponding script button.
        """
        # Navigatorfree software and manual download:
        # https://navigatorfree.mapfactor.com/de/?force_isolation=true
        # https://www.mapfactor.com/manuals/?force_isolation=true

        # Create an additional measurement thread to keep the GUI alive.
        # Please change 'MyUserScript._run_measurement' according to your class name.
        # With "args=[param1, parm2, paramn]" you can pass some optional parameters to the measurement thread.
        meas_dlg = DFAInfoDlg(self.main_gui)
        mthread = MeasDlgThread(main_gui=self.main_gui, dlg=meas_dlg,
                                callback=DFAutopilot._run_measurement,
                                args=[self.signalshark.addr])

        # Start the measurement thread and show a visualization (progress) dialog.
        mthread.start_measurement()
        # The lines of code below will be executed after the measurement thread has done it's job.
        # ...

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         signalshark_addr: str):
        """ Main method of the measurement thread that handles a time consuming measurement.

        Please do NOT use any wxPython elements like a MessageBox or a Dialog here!
        They are only permitted in the GUI thread ('_run_script' method).
        Please use the "update_progress" method to display information!

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param signalshark_addr: Custom parameter to handover the ip address of the SignalShark
        :return:
        """
        update_progress(msg="Checking measurement settings.", show_progress=False)
        # Setup connection to the SignalShark and display error message if connection fails.
        # -----------------------------------------------------------
        signalshark = SignalSharkDev(signalshark_addr)
        if not signalshark.connect():
            return wait_for_stopevent("Cannot connect to the SignalShark device!", icon_style=wx.ICON_ERROR)

        nav_dev: Optional[INavDevBase] = None
        settings: Optional[SettingsData] = None
        try:
            # Add a Auto DF Task if it does not already exist.
            if not signalshark.scpi.check_add_task(TaskTypes.AUTO_DF):
                # Check if desired view exists and add one if it does not.
                signalshark.scpi.check_add_view(ViewTypes.SPECTRUM)
                signalshark.scpi.check_add_view(ViewTypes.BEARING, 1, ViewDirections.RIGHT)
                signalshark.scpi.check_add_view(ViewTypes.MAP, 1, ViewDirections.BELOW)
                return wait_for_stopevent("Please configure the newly added task (e.g. frequency range)",
                                          icon_style=wx.ICON_INFORMATION)

            # Check required views:
            bearing_exists, bearing_index = signalshark.scpi.check_add_view(ViewTypes.BEARING, 1, ViewDirections.RIGHT)
            map_exists, map_index = signalshark.scpi.check_add_view(ViewTypes.MAP, 1, ViewDirections.BELOW)
            if not bearing_exists or not map_exists:
                return wait_for_stopevent("Please configure the added Views for localization.",
                                          icon_style=wx.ICON_INFORMATION)

            # Get Settings from XML
            # -----------------------------------------------------------
            settings = fh.load_dclass_from_xml(SettingsData, fh.LAST_SETTINGS_FILENAME)
            if settings is None:
                settings = SettingsData()

            # Setup connection to the navigation device
            # -----------------------------------------------------------
            nav_devs = fh.get_navdevs()
            nav_dev_cls = nav_devs.get(settings.nav_dev_name)
            if nav_dev_cls is None:
                return wait_for_stopevent("Please select a navigation device in the settings menu first!",
                                          icon_style=wx.ICON_INFORMATION)

            nav_dev_ip = ""
            update_progress(msg="Establishing the connection to the navigation device...", show_progress=True, force_update=True)
            if settings.nav_dev_auto_ip:
                nic_list = nt.get_nic_list()
                if not settings.nav_dev_nic in nic_list:
                    return wait_for_stopevent("Please select a valid network interface card in the settings menu first!",
                                              icon_style=wx.ICON_INFORMATION)

                nic_ip = settings.nav_dev_nic.split(";")[0]
                ip_list = nt.scan_ip4_network(stopevent, nic_ip, settings.nav_dev_port, 1.0, 1, 20, 30.0,
                                              [nav_dev_cls], DFAutopilot.ip_request)
                if len(ip_list) > 0 and ip_list[0] is not None:
                    nav_dev_ip = ip_list[0]
                else:
                    return wait_for_stopevent("The IP address of the navigation device could not be determined automatically!",
                                              icon_style=wx.ICON_INFORMATION)
            else:
                # Manual IP
                nav_dev_ip = settings.nav_dev_man_ip

            nav_dev = nav_dev_cls(nav_dev_ip, settings.nav_dev_port)
            if not nav_dev.connect():
                return wait_for_stopevent("Cannot connect to the navigation device!", icon_style=wx.ICON_ERROR)

            # Update the dialog message text and show progress bar
            last_tx_pos = GnssCoordinate()
            if settings.pos_hysteresis_perc > 0:
                hysteresis_factor = 1.0/settings.pos_hysteresis_perc
            else:
                hysteresis_factor = 0.2
            while True:
                # Periodically check, whether the user has clicked the cancel button,
                # and exit thread if so.
                if stopevent.is_set():
                    return

                update_progress(msg="Waiting for localization results...", show_progress=True, force_update=True)

                if not signalshark.connect():
                    stopevent.wait(settings.update_rate)
                    continue

                # Readout data from device:
                gnss_data = signalshark.scpi.sense.get_gnss_data()
                loc_result = signalshark.scpi.map.get_localization_data()
                signalshark.disconnect()

                if not geocalc.check_gnss(gnss_data):
                    stopevent.wait(settings.update_rate)
                    continue

                if not loc_result.valid:
                    stopevent.wait(settings.update_rate)
                    continue

                # Calculate distance to last localization:
                actual_tx_pos = GnssCoordinate(loc_result.coordinate.latitude, loc_result.coordinate.longitude)
                distance_last_loc = fabs(geocalc.get_distance_haversine(last_tx_pos, actual_tx_pos))

                # Calculate distance hysteresis:
                actual_ant_pos = GnssCoordinate(gnss_data.latitude, gnss_data.longitude)
                distance_to_antenna = fabs(geocalc.get_distance_haversine(actual_tx_pos, actual_ant_pos))
                distance_hysteresis = distance_to_antenna * hysteresis_factor
                if distance_hysteresis < settings.min_distance_hysteresis:
                    distance_hysteresis = settings.min_distance_hysteresis
                elif distance_hysteresis > settings.max_distance_hysteresis:
                    distance_hysteresis = settings.max_distance_hysteresis

                if distance_last_loc >= distance_hysteresis:
                    last_tx_pos = GnssCoordinate(actual_tx_pos.latitude, actual_tx_pos.longitude)
                    nav_result = nav_dev.set_destination(actual_tx_pos)
                    if nav_result:
                        update_progress(msg="The target has been updated.", show_progress=True)
                    else:
                        return wait_for_stopevent("The Navigation device reported an error: {}".format(nav_result),
                                                  icon_style=wx.ICON_ERROR)

                # Allow the thread to synchronize the stop event.
                stopevent.wait(settings.update_rate)
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e), icon_style=wx.ICON_ERROR)
        finally:
            # Close connection.
            if signalshark is not None:
                signalshark.disconnect()

            if nav_dev is not None:
                if settings is not None and settings.nav_auto_stop:
                    nav_dev.stop_navigation()
                nav_dev.disconnect()


    @staticmethod
    def ip_request(nav_dev_ip: str, nav_dev_port: int, socket_timeout: float, args: list) -> str:
        """Checks if network device is a navigation system.

        :param nav_dev_ip: IP address of the network device to check.
        :param nav_dev_port: Port of the network device to check.
        :param socket_timeout: Timeout time for the socket connection.
        :param args: Additional parameters.
        :return: Returns the IP address or an empty string.
        """
        nav_dev: Optional[INavDevBase] = None
        try:
            nav_dev_cls = args[0]
            nav_dev = nav_dev_cls(nav_dev_ip, nav_dev_port)
            if not nav_dev.connect():
                return ""
            version = nav_dev.get_remote_software_version()
            if version:
                return nav_dev_ip
            return ""
        except:
            return ""
        finally:
            if nav_dev is not None:
                nav_dev.disconnect()

