#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import time
import datetime as DT
import importlib
import inspect
from dateutil import tz
from typing import Optional, List, Callable, TypeVar, Type
import dataclasses
from dataclasses import dataclass, fields
import copy
from pathlib import Path
import os
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
from nardascripting.tools import xmlhandler as xml
from .navdev import INavDevBase


T = TypeVar("T")


LAST_SETTINGS_FILENAME = "dfautopilot_settings.xml"


def get_file_path(filename: str) -> Path:
    """Gets the full path of the given filename.

    :param filename: The name of the file.
    :return: Returns the path of the file.
    """
    return Path(__file__).parent.joinpath("data\\" + filename)


def save_dclass_as_xml(dclass: dataclasses.dataclass, filename: str) -> bool:
    """Saves a data class to an xml file.

    :param dclass: The data class to save.
    :param filename: The name of the json file.
    :return: Returns True if succeeded.
    """
    try:
        file_path = get_file_path(filename)
        return xml.save_as_xml_file(dclass, str(file_path))
    except Exception as e:
        return False


def load_dclass_from_xml(cls: Type[T], filename: str) -> Optional[T]:
    """Loads a data class from a xml file in the data folder

    :param cls: The type of the data class  to return.
    :param filename: The name of the json file.
    :return: Returns the dataclass or None
    """
    try:
        file_path = get_file_path(filename)
        return xml.load_from_xml_file(cls, str(file_path))
    except Exception as e:
        return None


def get_navdevs() -> dict:
    """Searches for navigation device clases

    :return: Returns dict containing available navdev classes or empty dict.
    """
    try:
        navdevs_dict = {}

        navdev_path = Path(__file__).parent.joinpath("navdevs")
        for f in list(navdev_path.glob('**/*.py')):
            filename = f.name
            if filename.startswith('__'):
                continue
            try:
                nav_dev_mod = importlib.import_module("." + f.stem, __package__ + ".navdevs")
                for item in inspect.getmembers(nav_dev_mod, inspect.isclass):
                    name = item[0]
                    dcls = item[1]
                    if name != "INavDevBase" and issubclass(dcls, INavDevBase):
                        navdevs_dict[dcls.get_name()] = dcls
            except Exception as err:
                pass
        return navdevs_dict
    except Exception as e:
        return {}


# This is just a python module containing helper classes/functions:
if __name__ == '__main__':
    pass