#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################

# https://kite.com/python/answers/how-to-find-the-distance-between-two-lat-long-coordinates-in-python

import math
from nardascripting.base.scpi.enums import GnssFix
from nardascripting.base.scpi.datatypes import GnssCoordinate, GnssDataRaw


def check_gnss(gnss: GnssDataRaw):
    """ Check if GNSS data exists and fix is better than "NOFIX".

    :param gnss:
    :return: Returns True, if GNSS data is OK.
    """
    if gnss:
        if gnss.fix:
            if gnss.fix != GnssFix.NOFIX:
                return True
    return False

def get_distance_haversine(coord1: GnssCoordinate, coord2: GnssCoordinate):
    """Calculates the distance between two gnss coordinates via haversine algorithm.

    :param coord1: GNSS coordinate 1 (lat, lon)
    :type coord1: nardascripting.base.scpi.datatypes.GnssCoordinate
    :param coord2: GNSS coordinate 2 (lat, lon)
    :type coord1: nardascripting.base.scpi.datatypes.GnssCoordinate
    :return: Distance in meter or None
    :rtype: float
    """
    R = 6372800  # Earth radius in meters
    result = None
    try:
        # Change to radians
        lat1_r = math.radians(coord1.latitude)
        lon1_r = math.radians(coord1.longitude)

        lat2_r = math.radians(coord2.latitude)
        lon2_r = math.radians(coord2.longitude)

        # Calculate lat lon distances
        delta_lon_r = lon2_r - lon1_r
        delta_lat_r = lat2_r - lat1_r

        # Apply Haversine formula
        a = math.sin(delta_lat_r / 2) ** 2 + math.cos(lat1_r) * math.cos(lat2_r) * math.sin(delta_lon_r / 2) ** 2
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
        result = R * c
    except:
        result = None

    return result


