#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import wx
from typing import Optional, List, Callable
from nardascripting.base.measthread import *
from nardascripting.base.scpi.enums import *
from .. dfadata import *
from .. import filehandling as fh
from .dfadialogsbase import *
from .images import scriptimages as appimg


class DFASettingsDlg(DFASettingsBaseDlg):
    def __init__(self, parent, nav_devs: dict, nic_list: List[str]):
        """Initialization"""
        super().__init__(parent)

        # Get Settings from XML
        self.settings = fh.load_dclass_from_xml(SettingsData, fh.LAST_SETTINGS_FILENAME)
        if self.settings is None:
            self.settings = SettingsData()

        # Python Propertygrid:
        # https://docs.wxpython.org/wx.propgrid.PGProperty.html

        # Color setup:
        # -----------------------------------------------------------
        self.pg_settings.SetCellBackgroundColour(wx.Colour(51, 51, 51))
        self.pg_settings.SetCellBackgroundColour(wx.Colour(51, 51, 51))
        self.pg_settings.SetCellTextColour(wx.Colour(255, 255, 255))
        self.pg_settings.SetCellDisabledTextColour(wx.Colour(102, 102, 102))
        self.pg_settings.SetLineColour(wx.Colour(102, 102, 102))
        self.pg_settings.SetCaptionBackgroundColour(wx.Colour(80, 80, 80))
        self.pg_settings.SetCaptionTextColour(wx.Colour(255, 255, 255))
        self.pg_settings.SetEmptySpaceColour(wx.Colour(51, 51, 51))
        self.pg_settings.SetMarginColour(wx.Colour(51, 51, 51))

        # Navigation device settings
        # -----------------------------------------------------------
        # Add items:
        self.pgi_cat_nav_dev = self.pg_settings.Append(
            pg.PropertyCategory(u"Navigation Device Settings", u"Navigation Device Settings"))

        self.nav_devs = nav_devs
        self.nav_dev_names = [*nav_devs]
        if not self.nav_dev_names or len(self.nav_dev_names) < 1:
            nav_dev_names = [""]
        self.nav_dev_name_num = list(range(len(self.nav_dev_names)))
        self.pgi_nav_dev_name = pg.EnumProperty(label=u"Navigation Device",
                                                name=u"Navigation Device",
                                                labels=self.nav_dev_names, values=self.nav_dev_name_num, value=0)
        self.pgi_nav_dev_name = self.pg_settings.Append(self.pgi_nav_dev_name)

        self.pgi_nav_dev_port = self.pg_settings.Append(pg.UIntProperty(u"Port", u"Port"))

        self.pgi_nav_dev_auto_ip = self.pg_settings.Append(
            pg.BoolProperty(u"Determine IP address automatically", u"Determine IP address automatically"))

        self.nic_list = nic_list
        if not self.nic_list or len(self.nic_list) < 1:
            self.nic_list = [""]
        self.nic_list_num = list(range(len(self.nic_list)))
        self.pgi_nav_dev_nic = pg.EnumProperty(label=u"Network Interface Card (NIC)",
                                               name=u"Network Interface Card (NIC)",
                                               labels=self.nic_list, values=self.nic_list_num, value=0)
        self.pgi_nav_dev_nic = self.pg_settings.Append(self.pgi_nav_dev_nic)

        self.pgi_nav_dev_man_ip = self.pg_settings.Append(
            pg.StringProperty(u"Manual IP Address", u"Manual IP Address"))

        # Localization settings:
        # -----------------------------------------------------------
        self.pgi_cat_general = self.pg_settings.Append(
            pg.PropertyCategory(u"Localization Settings", u"Localization Settings"))

        self.pgi_min_distance_hysteresis = self.pg_settings.Append(
            pg.UIntProperty(u"Minimum distance hysteresis (m)",
                            u"Minimum distance hysteresis (m)"))
        self.pgi_min_distance_hysteresis.SetAttribute("Min", 10)

        self.pgi_max_distance_hysteresis = self.pg_settings.Append(
            pg.UIntProperty(u"Maximum distance hysteresis (m)",
                            u"Maximum distance hysteresis (m)"))
        self.pgi_max_distance_hysteresis.SetAttribute("Min", 100)

        self.pgi_pos_hysteresis_perc = self.pg_settings.Append(
            pg.UIntProperty(u"Localization hysteresis in percent (%)",
                            u"Localization hysteresis in percent (%)"))
        self.pgi_pos_hysteresis_perc.SetEditor("SpinCtrl")
        self.pgi_pos_hysteresis_perc.SetAttribute("Min", 1)
        self.pgi_pos_hysteresis_perc.SetAttribute("Max", 60)

        self.pgi_update_rate = self.pg_settings.Append(
            pg.UIntProperty(u"Update rate (s)",
                            u"Update rate (s)"))
        self.pgi_update_rate.SetEditor("SpinCtrl")
        self.pgi_update_rate.SetAttribute("Min", 5)
        self.pgi_update_rate.SetAttribute("Max", 300)

        self.pgi_nav_auto_stop = self.pg_settings.Append(
            pg.BoolProperty(u"Stop navigation on close", u"Stop navigation on close"))


        # Set item values:
        # -----------------------------------------------------------
        try:
            nav_dev_name_ind = self.nav_dev_names.index(self.settings.nav_dev_name)
            self.pg_settings.SetPropertyValue(self.pgi_nav_dev_name, nav_dev_name_ind)
        except:
            pass

        self.pg_settings.SetPropertyValue(self.pgi_nav_dev_port, self.settings.nav_dev_port)
        self.pg_settings.SetPropertyValue(self.pgi_nav_dev_auto_ip, self.settings.nav_dev_auto_ip)

        try:
            nav_dev_nic_ind = self.nic_list.index(self.settings.nav_dev_nic)
            self.pg_settings.SetPropertyValue(self.pgi_nav_dev_nic, nav_dev_nic_ind)
        except:
            pass

        self.pg_settings.SetPropertyValue(self.pgi_nav_dev_man_ip, self.settings.nav_dev_man_ip)
        self.pg_settings.SetPropertyValue(self.pgi_min_distance_hysteresis, self.settings.min_distance_hysteresis)
        self.pg_settings.SetPropertyValue(self.pgi_max_distance_hysteresis, self.settings.max_distance_hysteresis)
        self.pg_settings.SetPropertyValue(self.pgi_pos_hysteresis_perc, self.settings.pos_hysteresis_perc)
        self.pg_settings.SetPropertyValue(self.pgi_update_rate, self.settings.update_rate)
        self.pg_settings.SetPropertyValue(self.pgi_nav_auto_stop, self.settings.nav_auto_stop)

    def on_ok_button(self, event):
        """OK button clicked.

        :param event: Dialog event trigger when save button is clicked
        """
        try:
            nav_dev_name_ind = self.pg_settings.GetPropertyValue(self.pgi_nav_dev_name)
            self.settings.nav_dev_name = self.nav_dev_names[nav_dev_name_ind]
            self.settings.nav_dev_port = self.pg_settings.GetPropertyValue(self.pgi_nav_dev_port)
            self.settings.nav_dev_auto_ip = self.pg_settings.GetPropertyValue(self.pgi_nav_dev_auto_ip)
            nav_dev_nic_ind = self.pg_settings.GetPropertyValue(self.pgi_nav_dev_nic)
            self.settings.nav_dev_nic = self.nic_list[nav_dev_nic_ind]
            self.settings.nav_dev_man_ip = self.pg_settings.GetPropertyValue(self.pgi_nav_dev_man_ip)
            self.settings.min_distance_hysteresis = self.pg_settings.GetPropertyValue(self.pgi_min_distance_hysteresis)
            self.settings.max_distance_hysteresis = self.pg_settings.GetPropertyValue(self.pgi_max_distance_hysteresis)
            self.settings.pos_hysteresis_perc = self.pg_settings.GetPropertyValue(self.pgi_pos_hysteresis_perc)
            self.settings.update_rate = self.pg_settings.GetPropertyValue(self.pgi_update_rate)
            self.settings.nav_auto_stop = self.pg_settings.GetPropertyValue(self.pgi_nav_auto_stop)

            fh.save_dclass_as_xml(self.settings, fh.LAST_SETTINGS_FILENAME)
        except:
            pass
        finally:
            event.Skip()


class DFAInfoDlg(PDFAInfoDlg):
    """This is the SignalShark Script Selector minimize window class"""

    def __init__(self, parent):
        # initialize parent class
        super().__init__(parent=parent)

        self.init_progress_bar(self.pgb_progress)
        self.init_dlg_buttons(self.sdbs_buttons, wx.OK)
        self.btn_ok.SetLabel("Stop Navigation")

        # Set images:
        widthx,heightx = self.bmp_title.GetSize()
        title_image = appimg.img_Navigation.GetImage()
        title_image = title_image.Scale(widthx, heightx, wx.IMAGE_QUALITY_HIGH)
        self.bmp_title.SetBitmap(wx.Bitmap(title_image))

        # Get Settings from XML
        self.settings = fh.load_dclass_from_xml(SettingsData, fh.LAST_SETTINGS_FILENAME)
        if self.settings is None:
            self.settings = SettingsData()

        self.__m_down = False
        self.__last_pos = wx.Point(0, 0)

        # Load start position
        width, height = wx.GetDisplaySize()
        app_size = self.GetSize()
        max_x = width - app_size.GetWidth()
        max_y = height - app_size.GetHeight()
        winpos_x = self.settings.win_last_pos_x
        winpos_y = self.settings.win_last_pos_y

        if winpos_x > (max_x):
            winpos_x = max_x

        if winpos_y > (max_y):
            winpos_y = max_y

        self.SetPosition(wx.Point(winpos_x, winpos_y))
        self.Bind(wx.EVT_CLOSE, self.OnClose)

    def __del__(self):
        pass

    def update_progress(self, evt: ProgressEvent):
        """Receives data from thread and updates progressbar and message text.

        :param evt: Event of type ProgressEvent.
        :type evt: ProgressEvent
        """

        # Handle progress
        self.update_progress_bar(evt, self.pgb_progress)

        # Handle message
        if evt.msg is not None:
            self.lbl_status.SetLabelText(evt.msg)

        # Handle data:
        if evt.data is not None:
            # data=[trigger_tstamp, trigger_source, trace_data_new]
            # Update Dialog Panel
            pass

        # Handle buttons:
        if evt.btn_style:
            self.update_btn_style(evt.btn_style)

        # Handle icons:
        if evt.icon_style:
            self.update_icon_style(self.bmp_icon, evt.icon_style)

        # Handle help text:
        if evt.help_text:
            self.help_text = evt.help_text

        # Update GUI
        self.Layout()
        # self.Refresh() # Causes this window, and all of its children recursively to be repainted.
        self.Update()  # Repaints the invalidated area

    def on_mouse_down(self, event):
        self.__last_pos = wx.GetMousePosition()
        self.__m_down = True
        event.Skip()

    def on_mouse_move(self, event):
        if self.__m_down:
            # self.ScreenToClient(event.GetEventObject().ClientToScreen(event.GetPos()))
            # pos = event.GetPosition()
            pos = wx.GetMousePosition()
            delt_x = pos.x - self.__last_pos.x
            delt_y = pos.y - self.__last_pos.y
            self.__last_pos = pos

            winpos = self.GetPosition()
            winpos.x += delt_x
            winpos.y += delt_y
            self.SetPosition(winpos)
        event.Skip()

    def on_mouse_up(self, event):
        self.__m_down = False
        event.Skip()

    def on_mouse_leave(self, event):
        self.__m_down = False
        event.Skip()

    def OnClose(self, event):
        """
        Handles the window closing event.

        Makes shure, that the connection to the device will be released.
        :param event: The closing event.
        """
        winpos = self.GetPosition()
        self.settings.win_last_pos_x = winpos.x
        self.settings.win_last_pos_y = winpos.y

        fh.save_dclass_as_xml(self.settings, fh.LAST_SETTINGS_FILENAME)
        event.Skip()