#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2022 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


"""
Navigation device remote control class
"""

import abc
from nardascripting.base.comm import *
from nardascripting.base.scpi.datatypes import GnssCoordinate
from ..navdev import *


class NavDevNavigator(INavDevBase):
    """ Navigatorfree software and manual download:
    https://navigatorfree.mapfactor.com/de/?force_isolation=true
    https://www.mapfactor.com/manuals/?force_isolation=true

    Remote commands:
    https://kb.mapfactor.com/kb/Navigator/RemoteCommands
    """

    def __init__(self, addr="127.0.0.1", port=4242, timeout=10):
        super(NavDevNavigator, self).__init__(addr, port, timeout)

    @staticmethod
    def get_name() -> str:
        """ Gets the name of the navigation system

        :return: Returns the name of the navigation system as string
        """
        return "Mapfactor Navigator"

    def get_remote_software_version(self) -> str:
        """Returns remote device application software version

        :return: Remote device application software version
        """
        retvalue = self.query("$software_version")
        if "error" in retvalue:
            self._errors.append("Error on get_remote_software_version!")
        return retvalue

    def set_display_text_message(self, msg: str, show_immediatelly: bool = True) -> str:
        """Shows a message on remote device

        :param msg: The message to show.
        :param show_immediatelly: If True, the message is shown immediately; placeholder icon is displayed otherwise
        :return: Returns an empty string if succeded or an error string.
        """
        if show_immediatelly:
            retvalue = self.query("$message=" + aq(msg) + ";instant")
        else:
            retvalue = self.query("$message=" + aq(msg))

        if not "OK" in retvalue:
            self._errors.append("Error on set_display_text_message!")
            return retvalue
        else:
            return ""

    def set_destination(self, destination: GnssCoordinate, name: str = None) -> str:
        """Sets new destination and starts navigation.

        :param destination: The desired gnss position.
        :param name: An optional name for the destination point.
        :return: Returns an empty string if succeded or an error string.
        """
        retvalue = self.query("$destination=clear")
        if name:
            retvalue = self.query("$destination=" + str(destination.latitude) +
                                      "," + str(destination.longitude) + "," + aq(name) + ";navigate")
        else:
            retvalue = self.query("$destination=" + str(destination.latitude) +
                                      "," + str(destination.longitude) + ";navigate")

        if not "OK" in retvalue:
            self._errors.append("Error on set_destination!")
            return retvalue
        else:
            return ""

    def stop_navigation(self) -> str:
        """Stops navigation on remote device

        :return: Returns an empty string if succeded or an error string.
        """
        retvalue = self.query("$stop_navigation")
        if not "OK" in retvalue:
            self._errors.append("Error on stop_navigation!")
            return retvalue
        else:
            return ""